@@ -17,10 +17,12 @@
 import collections
 import datetime
 import struct
+import sys
+import traceback
 
 import bson
 from bson.codec_options import CodecOptions
-from bson.py3compat import itervalues, string_type, iteritems, u
+from bson.py3compat import itervalues, string_type, iteritems
 from bson.son import SON
 from pymongo import ASCENDING
 from pymongo.errors import (CursorNotFound,
@@ -28,6 +30,7 @@ from pymongo.errors import (CursorNotFound,
                             ExecutionTimeout,
                             NotMasterError,
                             OperationFailure,
+                            ProtocolError,
                             WriteError,
                             WriteConcernError,
                             WTimeoutError)
@@ -35,7 +38,10 @@ from pymongo.message import _Query, _convert_exception
 from pymongo.read_concern import DEFAULT_READ_CONCERN
 
 
-_UUNDER = u("_")
+_UUNDER = u"_"
+
+_UNICODE_REPLACE_CODEC_OPTIONS = CodecOptions(
+    unicode_decode_error_handler='replace')
 
 
 def _gen_index_name(keys):
@@ -86,7 +92,9 @@ def _index_document(index_list):
     return index
 
 
-def _unpack_response(response, cursor_id=None, codec_options=CodecOptions()):
+def _unpack_response(response,
+                     cursor_id=None,
+                     codec_options=_UNICODE_REPLACE_CODEC_OPTIONS):
     """Unpack a response from the database.
 
     Check the response for errors and unpack, returning a dictionary
@@ -106,7 +114,8 @@ def _unpack_response(response, cursor_id=None, codec_options=CodecOptions()):
     response_flag = struct.unpack("<i", response[:4])[0]
     if response_flag & 1:
         # Shouldn't get this response if we aren't doing a getMore
-        assert cursor_id is not None
+        if cursor_id is None:
+            raise ProtocolError("No cursor id for getMore operation")
 
         # Fake a getMore command response. OP_GET_MORE provides no document.
         msg = "Cursor not found, cursor id: %d" % (cursor_id,)
@@ -240,8 +249,8 @@ def _first_batch(sock_info, db, coll, query, ntoreturn,
                  slave_ok, codec_options, read_preference, cmd, listeners):
     """Simple query helper for retrieving a first (and possibly only) batch."""
     query = _Query(
-        0, db, coll, 0, ntoreturn, query, None,
-        codec_options, read_preference, 0, 0, DEFAULT_READ_CONCERN)
+        0, db, coll, 0, query, None,
+        codec_options, read_preference, ntoreturn, 0, DEFAULT_READ_CONCERN)
 
     name = next(iter(cmd))
     duration = None
@@ -330,3 +339,20 @@ def _fields_list_to_dict(fields, option_name):
 
     raise TypeError("%s must be a mapping or "
                     "list of key names" % (option_name,))
+
+
+def _handle_exception():
+    """Print exceptions raised by subscribers to stderr."""
+    # Heavily influenced by logging.Handler.handleError.
+
+    # See note here:
+    # https://docs.python.org/3.4/library/sys.html#sys.__stderr__
+    if sys.stderr:
+        einfo = sys.exc_info()
+        try:
+            traceback.print_exception(einfo[0], einfo[1], einfo[2],
+                                      None, sys.stderr)
+        except IOError:
+            pass
+        finally:
+            del einfo
