@@ -22,7 +22,7 @@ import sys
 sys.path[0:0] = [""]
 
 from bson.code import Code
-from bson.py3compat import u, PY3
+from bson.py3compat import PY3
 from bson.son import SON
 from pymongo import (MongoClient,
                      monitoring,
@@ -30,9 +30,7 @@ from pymongo import (MongoClient,
                      DESCENDING,
                      ALL,
                      OFF)
-from pymongo.command_cursor import CommandCursor
 from pymongo.cursor import CursorType
-from pymongo.cursor_manager import CursorManager
 from pymongo.errors import (InvalidOperation,
                             OperationFailure,
                             ExecutionTimeout)
@@ -132,7 +130,7 @@ class TestCursorNoConnect(unittest.TestCase):
         self.assertEqual(0, cursor._Cursor__query_flags)
 
         # Exhaust - which mongos doesn't support
-        if not self.db.client.is_mongos:
+        if client_context.client is not None and not self.db.client.is_mongos:
             cursor = self.db.test.find(cursor_type=CursorType.EXHAUST)
             self.assertEqual(64, cursor._Cursor__query_flags)
             cursor2 = self.db.test.find().add_option(64)
@@ -228,7 +226,7 @@ class TestCursor(IntegrationTest):
         listener = EventListener()
         listener.add_command_filter('killCursors')
         saved_listeners = monitoring._LISTENERS
-        monitoring._LISTENERS = monitoring._Listeners([])
+        monitoring._LISTENERS = monitoring._Listeners([], [], [], [])
         coll = single_client(
             event_listeners=[listener])[self.db.name].pymongo_test
         results = listener.results
@@ -807,7 +805,7 @@ class TestCursor(IntegrationTest):
 
         self.assertEqual(3, db.test.find().where('this.x < 3').count())
         self.assertEqual(10, db.test.find().count())
-        self.assertEqual(3, db.test.find().where(u('this.x < 3')).count())
+        self.assertEqual(3, db.test.find().where(u'this.x < 3').count())
         self.assertEqual([0, 1, 2],
                          [a["x"] for a in
                           db.test.find().where('this.x < 3')])
@@ -1217,38 +1215,6 @@ class TestCursor(IntegrationTest):
         next(cursor)
         self.assertRaises(InvalidOperation, cursor.comment, 'hello')
 
-    def test_cursor_transfer(self):
-
-        # This is just a test, don't try this at home...
-
-        client = client_context.rs_or_standalone_client
-        db = client.pymongo_test
-
-        db.test.delete_many({})
-        db.test.insert_many([{'_id': i} for i in range(200)])
-
-        class CManager(CursorManager):
-            def __init__(self, client):
-                super(CManager, self).__init__(client)
-
-            def close(self, dummy, dummy2):
-                # Do absolutely nothing...
-                pass
-
-        client.set_cursor_manager(CManager)
-        self.addCleanup(client.set_cursor_manager, CursorManager)
-        docs = []
-        cursor = db.test.find().batch_size(10)
-        docs.append(next(cursor))
-        cursor.close()
-        docs.extend(cursor)
-        self.assertEqual(len(docs), 10)
-        cmd_cursor = {'id': cursor.cursor_id, 'firstBatch': []}
-        ccursor = CommandCursor(cursor.collection, cmd_cursor,
-                                cursor.address, retrieved=cursor.retrieved)
-        docs.extend(ccursor)
-        self.assertEqual(len(docs), 200)
-
     def test_modifiers(self):
         cur = self.db.test.find()
         self.assertTrue('$query' not in cur._Cursor__query_spec())
